import es
import gamethread
import popuplib
import usermsg
import msglib
import cfglib


info = es.AddonInfo()
info.author      = "AbSoLuTe"
info.name        = "Tell Spectator names"
info.basename    = "tellspec"
info.version     = "2.0"
info.url         = "http://addons.eventscripts.com/addons/view/%s"%info.basename
info.description = "Shows who is spectating whom" 

es.ServerVar(info.basename, info.version, info.description + " made by " + info.author).makepublic()


config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/' + info.basename + '.cfg')
config.text('%s by %s , Version: %s'% (info.basename,info.author,info.version))
config.text(info.description)
config.text('Visit %s for updates' % info.url)
config.text('Settings:')
config.text('\n')

auto_enable = config.cvar('auto_enable', 1, 'If the menu should be enabled on join')
updaterate = config.cvar('updaterate', 1, 'How often the menu should update (in seconds)')
style = config.cvar('style', 2, 'The style the spectators are shown ( 1 = hud hint, 2 = popup, 3 = toptext [You can set the updaterate pretty high with toptext])')
max_length = config.cvar('max_length', 8, 'The maximum length a name may have (Will be shortened otherwise: Absolute123 would be shortened to Absolu..)')
advert_time = config.cvar('advert_time', 120, 'The time between the adverts (in seconds)')
ignore_list = config.cvar('ignore_list', 'STEAM_0:0:44884;STEAM:0:123456789', 'These players will not been shown as spectating')
config.text('\n')
config.text('Following are the colors of the toptext (if its used):')
R = config.cvar('R', 175)
G = config.cvar('G', 175)
B = config.cvar('B', 175)
config.write()

config.execute()

enabled = []
spectargets = {}

def load():
    es.addons.registerClientCommandFilter(cmdFilter)
    es.regsaycmd('!disable',"%s/disabled"%info.basename,"Disables or Enables the Menu")
    for uid in es.getUseridList():
        player_connect({'userid':uid}) # fire the connect event on all players on the server
    
def unload():
    es.addons.unregisterClientCommandFilter(cmdFilter)
    es.unregsaycmd('!disable')
    gamethread.cancelDelayed("checkloop")
    gamethread.cancelDelayed("advertloop")
    del spectargets
    del enabled
    
def cmdFilter(userid,args):
    """ on spec_next or spec_mode or spec_prev the function GetNewTarget() will be called"""
    if args[0] == 'spec_next' or args[0] == 'spec_mode' or args[0] == 'spec_prev':
        gamethread.cancelDelayed("cmdThread%s" % userid) # use delayedname to stop it from calling GetNewTarget() excessivly
        gamethread.delayedname(0.1,"cmdThread%s" % userid,GetNewTarget,(int(userid))) # delay it by 0.1 or it will be too fast otherwise
        
    return True
    
def disabled():
    uid = int(es.getcmduserid())
    if uid in enabled:
        es.tell(uid,"#multi","#default[Spectator Menu] #greenYou will no longer see the spectator menu")
        enabled.remove(uid)
    else:
        es.tell(uid,"#multi","#default[Spectator Menu] #greenYou see the spectator menu again")
        enabled.append(uid)

advert = 0
def advertloop():
    global advert
    advert += 1
    if advert == 1:
        es.msg("#multi","#default[Spectator Menu] #greenType #lightgreen'!disable'#green to toggle the spectator menu.")
    else:
        es.msg("#multi","#default[Spectator Menu] #greenWritten by #defaultAbSoLuTe#green.")
        advert = 0
    gamethread.delayedname(advert_time,'advertloop',advertloop)
        
def round_start(event_var):
    gamethread.cancelDelayed("checkloop")
    checkloop()
        
def player_connect(event_var):
    """ on connect add the players to the dict and make [0] a list and [1] an int """
    uid = int(event_var['userid'])
    spectargets[uid] = [[],0] 
    if auto_enable:
        enabled.append(uid)
        
def player_disconnect(event_var):
    """ On disconnect search new targets for players in the leaving players list """
    uid = int(event_var['userid'])
    if uid in enabled:
        enabled.remove(uid)
        
    for uid2 in spectargets[uid][0]:
        gamethread.delayed(0.1,GetNewTarget,(uid2))
    if uid in spectargets:
        del spectargets[uid]

def player_spawn(event_var):
    """ On spawn reset the current target and remove the userid from their current speclist """
    userid = int(event_var["userid"])
    if spectargets[userid][1] != 0:
        if userid in spectargets[spectargets[userid][1]][0]:
            spectargets[spectargets[userid][1]][0].remove(userid)
    spectargets[userid][1] = 0
    
def player_death(event_var):
    """ On death loop through the spectators of the dying player and search new targets """
    userid = int(event_var["userid"])
    for uid in spectargets[userid][0]:
        gamethread.delayed(0.1,GetNewTarget,(uid))
    gamethread.delayed(0.1,GetNewTarget,(userid))
    
def GetNewTarget(userid):
    """ Just a simple function to get the current target of userid and append them to the list of their target"""
    if not userid:
        return
    if int(es.getplayerprop(userid, "CCSPlayer.baseclass.m_iObserverMode")) in (1,3,4):
        uid = getSpectargetByIndex(es.getplayerprop(userid, 'CBasePlayer.m_hObserverTarget'))
        if uid != -1:
            if spectargets[userid][1] != 0: # if the old target is not equal 0
                if userid in spectargets[spectargets[userid][1]][0]: # check if the userid is in the list of the old target
                    spectargets[spectargets[userid][1]][0].remove(userid) # remove it then
            spectargets[userid][1] = uid # set the current target for the userid to the new target
            spectargets[uid][0].append(userid) # append the userid to the list of the new target
    else:
        if spectargets[userid][1] != 0:
            if userid in spectargets[spectargets[userid][1]][0]:
                spectargets[spectargets[userid][1]][0].remove(userid)
            spectargets[userid][1] = 0 # set the current target to zero and remove the userid from the old speclist, because he is now in free mode
        
def getSpectargetByIndex(index):
    """ Loops through alive targets to compare the given index and the handle of the looped player """
    for userid in filter(lambda temp: not es.getplayerprop(temp, "CCSPlayer.baseclass.pl.deadflag"), es.getUseridList()):
        if es.getplayerhandle(userid) == index:
            return userid
            
    return -1 # if it cant find the userid return -1
    
def checkloop():
    """ Loop through alive players to give them their spectator menu """
    for uid in filter(lambda temp: not es.getplayerprop(temp, "CCSPlayer.baseclass.pl.deadflag"), es.getUseridList()):
        spectators = ""
        telllist = []
        for specs in spectargets[uid][0]:
            if es.getplayersteamid(specs) in str(ignore_list).split(";"):
                continue
            if specs in enabled:
                telllist.append(specs) # add the spectators of uid to a list
            spectators = addName(spectators,specs)
            
        if spectators:
            if uid in enabled: 
                if style == 3:
                    msglib.VguiDialog('Your Spectators: ' + spectators[1:],'',5,"%s %s %s 255" % (R,B,G),updaterate).send(uid)
                elif style == 2:
                    popuplib.quicksend(updaterate, uid, "Your Spectators:" + spectators + "\nType !disable to disable")
                elif style == 1:
                    usermsg.hudhint(uid,"Your Spectators:\n" + spectators[1:])
            if telllist:
                """ loop through the telllist to send the message to all the spectators of uid """
                for uid2 in telllist: 
                    if style == 3:
                        msglib.VguiDialog('Spectating %s: ' % es.getplayername(uid) + spectators[1:],'',5,"%s %s %s 255" % (R,B,G),updaterate).send(uid2)
                    elif style == 2:
                        popuplib.quicksend(updaterate, uid2, 'Spectating %s: ' % es.getplayername(uid) + spectators + "\nType !disable to disable")
                    elif style == 1:
                        usermsg.hudhint(uid2,'Spectating %s:\n ' % es.getplayername(uid) + spectators[1:])
          
    gamethread.delayedname(updaterate,"checkloop",checkloop)
   
def addName(spectators,specs):
    """ A function to add the name of the user specs to the string spectators depending on style """
    if len(es.getplayername(specs)) > max_length:
        realname = es.getplayername(specs)[:max_length-2] + ".."
    else:
        realname = es.getplayername(specs)
    if style == 3:
        spectators = spectators + ", " + realname.replace("\n","\\n") # the replace doesnt work :(
    elif style == 2:
        spectators = spectators + "\n  " + realname.replace("\n","\\n")
    elif style == 1:
        temp = temp + 1
        if temp == 4:
            temp = 0
            spectators = spectators + "\n" + realname.replace("\n","\\n")
        else:
            spectators = spectators + ", " + realname.replace("\n","\\n")
    return spectators
    

advertloop()